% Codes for the paper: "Sovereign Default: The Role of Expectations" by
% Joao Ayres (University of Minnesota and FRB-Minneapolis),
% Gaston Navarro (New York University),
% Juan Pablo Nicolini (FRB-Minneapolis and Universidad Di Tella), and
% Pedro Teles (Banco de Portugal, Universidade Catolica Portuguesa, and
% CEPR).
% Written by Joao Ayres and Gaston Navarro
% Date: June 2015

% 2 period model (competitive case)
% Bimodal distribution

clear, clc

%% Plot expected return

Rs = 1.2;          % risk-free rate
p = 0.2;           % probabiblity of drawing from distribution 1
bplot = [2.0 2.25 2.5];
gridR = linspace(0.5,3,200);
grid1 = zeros(1,length(gridR));
grid2 = zeros(1,length(gridR));
grid3 = zeros(1,length(gridR));
constR = Rs*ones(1,length(gridR));
for i = 1:length(gridR)
    grid1(i) = gridR(i)*(1 - p*normcdf(1.0+gridR(i)*bplot(1),4,0.1) - (1-p)*normcdf(1.0+gridR(i)*bplot(1),6,0.1));
    grid2(i) = gridR(i)*(1 - p*normcdf(1.0+gridR(i)*bplot(2),4,0.1) - (1-p)*normcdf(1.0+gridR(i)*bplot(2),6,0.1));
    grid3(i) = gridR(i)*(1 - p*normcdf(1.0+gridR(i)*bplot(3),4,0.1) - (1-p)*normcdf(1.0+gridR(i)*bplot(3),6,0.1));    
end

% Figure 4: Expected return for bimodal distribution
figure
plot(gridR,grid1,'b','LineWidth',3.5), hold on
plot(gridR,grid2,'--k','LineWidth',3.5)
plot(gridR,grid3,':r','LineWidth',3.5,'MarkerSize',20.0)
leg = legend(' $b_L$',' $b_M$',' $b_H$');
set(leg,'Interpreter','LaTex','Fontsize',24,'Location','NorthWest')         
legend boxoff
plot(gridR,constR,'-k','LineWidth',2)
xlim([0.9 2.6])
ylim([0.8 2.0])
xlabel('$R$','Interpreter','LaTex','Fontsize',30)
ylabel('$h(R;b)$','Interpreter','LaTex','Fontsize',30)
text(0.92,1.70,'$b_L<b_M<b_H$','Interpreter','LaTex','Fontsize',24)
text(0.92,1.22,'$R^{\ast}$','Interpreter','LaTex','Fontsize',24)
set(gca,'XGrid','off','YGrid','off','Fontsize',20);
set(gca,'xtick',[])
set(gca,'xticklabel',[])
set(gca,'ytick',[])
set(gca,'yticklabel','[]')



%% 2 period model (competitive case)
% Bimodal distribution (2 normal distributions)

% INPUTS
gamma = 2;         % risk aversion parameter
beta = 0.96;       % discount factor
w = 0.32;          % initial wealth
yd = 1;            % consumption in autarky
Rs = 1.2;          % risk-free rate
Bbar = 5;          % borrowing limit
mu1 = 4.0;         % distribution 1 (mean)
sigma1 = 0.1;      % distribution 1 (s.d.)
mu2 = 7;           % distribution 2 (mean)
sigma2 = 0.1;      % distribution 2 (s.d.)
p = 0.4;           % probabiblity of drawing from distribution 1
Rmax = 6;          % maximum interest rate in the grid
Nr = 1000;         % size of grid for interest rates
Robj = 4;          % interest rate for the objective function plot
Bgridmax = 10;     % maximum borrowing level for the objective function plot
Bgridmin = 0;      % minimum borrowing level for the objective function plot

%% SOLUTION

% Discretizing state space (endowment process)

% Extreme
m = 4; 
ymin1 = mu1 - m*sigma1;
ymax1 = mu1 + m*sigma1;
ymin2 = mu2 - m*sigma2;
ymax2 = mu2 + m*sigma2;

% Grids
Ny1 = 50;          % size of grid around mu1
Ny2 = 50;          % size of grid around mu2
Ny3 = 20;          % size of grid between both normal distributions
grid1 = linspace(ymin1,ymax1,Ny1);
grid2 = linspace(ymin2,ymax2,Ny2);
grid3 = zeros(1,Ny3);
for i = 1:Ny3
    grid3(i) = ymax1 + (i/(Ny3+1))*(ymin2-ymax1);
end

% Final grid
Ny = Ny1+Ny2+Ny3;
y_grid = zeros(1,Ny);

for i = 1:Ny
    if i <= Ny1
        y_grid(i) = grid1(i);
    elseif (i>Ny1) && (i <= Ny1+Ny3)
        y_grid(i) = grid3(i - Ny1);
    else
        y_grid(i) = grid2(i - (Ny1+Ny3));
    end
end

% Transition Matrix          
Q = zeros(Ny,1);
Q(1) = p*normcdf((((y_grid(1)+y_grid(2))/2)-mu1)/sigma1,0,1)+(1-p)*normcdf((((y_grid(1)+y_grid(2))/2)-mu2)/sigma2,0,1);
Q(end) = 1 - (p*normcdf((((y_grid(end)+y_grid(end-1))/2)-mu1)/sigma1,0,1)+(1-p)*normcdf((((y_grid(end)+y_grid(end-1))/2)-mu2)/sigma2,0,1));
for i = 2:(Ny-1)
    aux1 = p*normcdf((((y_grid(i)+y_grid(i+1))/2)-mu1)/sigma1,0,1)+(1-p)*normcdf((((y_grid(i)+y_grid(i+1))/2)-mu2)/sigma2,0,1);
    aux2 = p*normcdf((((y_grid(i)+y_grid(i-1))/2)-mu1)/sigma1,0,1)+(1-p)*normcdf((((y_grid(i)+y_grid(i-1))/2)-mu2)/sigma2,0,1);
    Q(i) = aux1 - aux2;
end

% Cumulative distribution
Qcdf = zeros(Ny,1);
Qcdf(1) = Q(1);
for i = 2:Ny
    Qcdf(i) = Qcdf(i-1)+Q(i);
end

% DEMAND FUNCTION

% Grids
Rdem = linspace(1.0,Rmax,Nr);
Rsup = linspace(Rs,Rmax,Nr);
Nb = 500;                               % grid for optimal borrowing
Bgrid = linspace(-w,Bbar,Nb);
Bgrid_val = zeros(Nb,1);
bdem = zeros(Nr,1);
alpha1 = (3.0-sqrt(5.0))/2.0;      	    % golden search parameter
alpha2 = (sqrt(5.0)-1)/2.0;				% golden search parameter
tol = 1.0e-09;

for i = 1:Nr
    for j = 1:Nb
        Bgrid_val(j) = util(Bgrid(j),Rdem(i),beta,w,yd,y_grid,Q,gamma);
    end
    [vz z] = max(Bgrid_val);
    a = Bgrid(z) - 1/Nb;
    b = min(Bgrid(z)+1/Nb,Bbar);   
    
    % GOLDEN SEARCH
    d = b-a;
    x1 = a + alpha1*d;
    f1 = util(x1,Rdem(i),beta,w,yd,y_grid,Q,gamma);
    x2 = a + alpha2*d;
    f2 = util(x2,Rdem(i),beta,w,yd,y_grid,Q,gamma);
    d = alpha1*alpha2*d;
    while (d > tol)
        d = d*alpha2;
         if (f2<f1)
            x2 = x1;
            f2 = f1;
            x1 = x1-d;
            f1 = util(x1,Rdem(i),beta,w,yd,y_grid,Q,gamma);
         else
            x1 = x2;
            f1 = f2;
            x2 = x2+d;
            f2 = util(x2,Rdem(i),beta,w,yd,y_grid,Q,gamma);
        end        
    end
    if f2 > f1
        bdem(i) = x2;
    else
        bdem(i) = x1;
    end
end

% SUPPLY FUNCTION

fsup = @(x,y) (Rs - y*(1 - p*normcdf(yd+y*x,mu1,sigma1) - (1-p)*normcdf(yd+y*x,mu2,sigma2)));

bsup = zeros(Nr,1);
for i = 1:Nr
    x1 = -15;
    x2 = 15;
    d = 1;
    while (d > tol)
        x = (x1+x2)/2;
        fx = fsup(x,Rsup(i));
        if (fx <= 0)
            x1 = x;
        else
            x2 = x;
        end
        d = abs(fx);
    end
    bsup(i) = x;
end

%% PLOTS

[max1 loc1] = max(bsup(1:50));
[max2 loc2] = max(bsup(51:end));
loc2 = 50+loc2;
[min3 loc3] = max(-bsup(loc1:loc2));
loc3 = loc1-1+loc3;

% Figure 5: Equilibrium borrowing and interest rate for the bimodal distribution
figure
plot(bsup(1:loc1),Rsup(1:loc1),'b','Linewidth',3.5), hold on
plot(bdem(1:152),Rdem(1:152),'--k','Linewidth',3.5)
plot(bdem(153:end),Rdem(153:end),'--k','Linewidth',3.5)
plot(bsup(loc1:loc3),Rsup(loc1:loc3),'r:','Linewidth',3.5)
plot(bsup(loc3:loc2),Rsup(loc3:loc2),'b','Linewidth',3.5)
plot(bsup(loc2:end),Rsup(loc2:end),'r:','Linewidth',3.5)
xlabel('$b$','Interpreter','LaTex','Fontsize',30)
ylabel('$R$','Interpreter','LaTex','Fontsize',30)
ylim([1.0 2.5])
xlim([0.8 3.2])
set(gca,'XGrid','off','YGrid','off','Fontsize',20);
set(gca,'xtick',[])
set(gca,'xticklabel',[])
set(gca,'ytick',[])
set(gca,'yticklabel','[]')
