% Codes for the paper: "Sovereign Default: The Role of Expectations" by
% Joao Ayres (University of Minnesota and Fed-Minneapolis),
% Gaston Navarro (New York University),
% Juan Pablo Nicolini (FRB-Minneapolis and Universidad Di Tella), and
% Pedro Teles (Banco de Portugal, Universidade Catolica Portuguesa, and
% CEPR).
% Written by Joao Ayres and Gaston Navarro
% Date: June 2015

% 2 period model (competitive case)
% Normal distribution

clear, clc

% 2 period model (competitive case)
% Normal distribution

% INPUTS
gamma = 5;         % risk aversion parameter
beta = 0.8;        % discount factor
w = -0.5;          % initial wealth
yd = 0.2;          % consumption in autarky
Rs = 1.04;         % risk-free rate
Bbar = 0.95;       % borrowing limit
mu = 1.5;          % distribution 1 (mean)
sigma = 0.2;       % distribution 1 (s.d.)
Rmax = 10;         % maximum interest rate in the grid
Nr = 1000;         % size of grid for interest rates
Robj = 1.5;        % interest rate for the objective function plot
Bgridmax = 10;     % maximum borrowing level for the objective function plot
Bgridmin = 0;      % minimum borrowing level for the objective function plot

%% SOLUTION

% Discretizing state space (endowment process)

% Extreme
m = 3.5; 
ymin = mu - m*sigma;
ymax = mu + m*sigma;

% Grids
Ny = 800;          % size of grid around mu1
y_grid = linspace(ymin,ymax,Ny);

% Transition Matrix          
Q = zeros(Ny,1);
Q(1) = normcdf((((y_grid(1)+y_grid(2))/2)-mu)/sigma,0,1);
Q(end) = 1 - normcdf((((y_grid(end)+y_grid(end-1))/2)-mu)/sigma,0,1);
for i = 2:(Ny-1)
    aux1 = normcdf((((y_grid(i)+y_grid(i+1))/2)-mu)/sigma,0,1);
    aux2 = normcdf((((y_grid(i)+y_grid(i-1))/2)-mu)/sigma,0,1);
    Q(i) = aux1 - aux2;
end

% Cumulative distribution
Qcdf = zeros(Ny,1);
Qcdf(1) = Q(1);
for i = 2:Ny
    Qcdf(i) = Qcdf(i-1)+Q(i);
end

% DEMAND FUNCTION

% Grids
Rdem = linspace(1.0,Rmax,Nr);
Rsup = linspace(Rs,Rmax,Nr);
Nb = 500;                               % grid for optimal borrowing
Bgrid = linspace(-w,Bbar,Nb);
Bgrid_val = zeros(Nb,1);
bdem = zeros(Nr,1);
alpha1 = (3.0-sqrt(5.0))/2.0;      	    % golden search parameter
alpha2 = (sqrt(5.0)-1)/2.0;				% golden search parameter
tol = 1.0e-9;

for i = 1:Nr

    a = -w;
    b = Bbar;
    
    % GOLDEN SEARCH
    d = b-a;
    x1 = a + alpha1*d;
    f1 = util(x1,Rdem(i),beta,w,yd,y_grid,Q,gamma);
    x2 = a + alpha2*d;
    f2 = util(x2,Rdem(i),beta,w,yd,y_grid,Q,gamma);
    d = alpha1*alpha2*d;
    while (d > tol)
        d = d*alpha2;
         if (f2<f1)
            x2 = x1;
            f2 = f1;
            x1 = x1-d;
            f1 = util(x1,Rdem(i),beta,w,yd,y_grid,Q,gamma);
         else
            x1 = x2;
            f1 = f2;
            x2 = x2+d;
            f2 = util(x2,Rdem(i),beta,w,yd,y_grid,Q,gamma);
        end        
    end
    if f2 > f1
        bdem(i) = x2;
    else
        bdem(i) = x1;
    end
end

% SUPPLY FUNCTION

fsup = @(x,y) (Rs - y*(1 - normcdf(yd+y*x,mu,sigma)));

bsup = zeros(Nr,1);
for i = 1:Nr
    x1 = -15;
    x2 = 15;
    d = 1;
    while (d > tol)
        x = (x1+x2)/2;
        fx = fsup(x,Rsup(i));
        if (fx <= 0)
            x1 = x;
        else
            x2 = x;
        end
        d = abs(fx);
    end
    bsup(i) = x;
end
[x loc] = max(bsup);

% Supply and demand in terms of a = bR
brsup = zeros(1,Nr);
brdem = zeros(1,Nr);
for i = 1:Nr
    brsup(i) = bsup(i)*Rsup(i);
    brdem(i) = bdem(i)*Rdem(i);
end

%% PLOTS

% Figure 2: Interest rate schedule R(b)
figure
plot(bsup(1:loc),Rsup(1:loc),'b','Linewidth',3.5), hold on
plot(bsup(loc:end),Rsup(loc:end),':r','Linewidth',3.5)
xlabel('$b$','Interpreter','LaTex','Fontsize',30)
ylabel('$R$','Interpreter','LaTex','Fontsize',30)
ylim([1.0 2])
xlim([0.6 0.95])
set(gca,'xtick',[])
set(gca,'xticklabel',[])
set(gca,'ytick',[])
set(gca,'yticklabel','[]')
set(gca,'XGrid','off','YGrid','off','Fontsize',20);

% Figure 3: Equilibrium borrowing and interest rate
figure
plot(bsup(1:loc),Rsup(1:loc),'b','Linewidth',3.5), hold on
plot(bdem,Rdem,'--k','Linewidth',3.5)
xlabel('$b$','Interpreter','LaTex','Fontsize',30)
ylabel('$R$','Interpreter','LaTex','Fontsize',30)
ylim([1.0 1.60])
xlim([0.75 0.95])
set(gca,'xtick',[])
set(gca,'xticklabel',[])
set(gca,'ytick',[])
set(gca,'yticklabel','[]')
plot(bsup(loc:end),Rsup(loc:end),':r','Linewidth',3.5)
set(gca,'XGrid','off','YGrid','off','Fontsize',20);

% Figure 6: Equilibrium outcomes with choice of b or a
figure
subplot(1,2,1)
plot(bsup(1:loc),Rsup(1:loc),'b','Linewidth',3.25), hold on
plot(bsup(loc:end),Rsup(loc:end),':r','Linewidth',3.25)
plot(bdem,Rdem,'k--','Linewidth',3.25)
xlabel('$b$','Interpreter','LaTex','Fontsize',30)
ylabel('$R$','Interpreter','LaTex','Fontsize',30) 
ylim([1.0 1.65])
xlim([0.7 0.95])
set(gca,'XGrid','off','YGrid','off','Fontsize',20)
set(gca,'xtick',[])
set(gca,'xticklabel',[])
set(gca,'ytick',[])
set(gca,'yticklabel','[]')
subplot(1,2,2)
plot(brsup(1:loc),Rsup(1:loc),'b','Linewidth',3.25), hold on
plot(brsup(loc:end),Rsup(loc:end),':r','Linewidth',3.25)
plot(brdem,Rdem,'k--','Linewidth',3.25)
xlabel('$a$','Interpreter','LaTex','Fontsize',30)
ylabel('$R$','Interpreter','LaTex','Fontsize',30)
ylim([1 1.65])
xlim([0.8 1.3])
set(gca,'XGrid','off','YGrid','off','Fontsize',20)
set(gca,'xtick',[])
set(gca,'xticklabel',[])
set(gca,'ytick',[])
set(gca,'yticklabel','[]')

%% Expected Return Functions
mu_plot = 4.7;
sigma_plot = 1.8;
Rs = 1.2;
ydef = 1;
bplot = [1.4 1.55 1.7];
gridR = linspace(0.8,5,100);
grid1 = zeros(1,length(gridR));
grid2 = zeros(1,length(gridR));
grid3 = zeros(1,length(gridR));
constR = Rs*ones(1,length(gridR));
for i = 1:length(gridR)
    grid1(i) = gridR(i)*(1-normcdf(ydef+bplot(1)*gridR(i),mu_plot,sigma_plot));
    grid2(i) = gridR(i)*(1-normcdf(ydef+bplot(2)*gridR(i),mu_plot,sigma_plot));
    grid3(i) = gridR(i)*(1-normcdf(ydef+bplot(3)*gridR(i),mu_plot,sigma_plot));
end

% Figure 1: Expected return h(R; b)
figure
plot(gridR,grid1,'b','LineWidth',3.5), hold on
plot(gridR,grid2,'--k','LineWidth',3.5)
plot(gridR,grid3,':r','LineWidth',3.5)
leg = legend(' $b_L$',' $b_M$',' $b_H$');
set(leg,'Interpreter','LaTex','Fontsize',24,'Location','NorthWest')         
legend boxoff
plot(gridR,constR,'-k','LineWidth',2)
xlim([0.8 3.5])
ylim([0.7 1.45])
xlabel('$R$','Interpreter','LaTex','Fontsize',30)
ylabel('$h(R;b)$','Interpreter','LaTex','Fontsize',30)
text(0.9,1.29,'$b_L<b_M<b_H$','Interpreter','LaTex','Fontsize',24)
text(0.9,1.22,'$R^{\ast}$','Interpreter','LaTex','Fontsize',24)
set(gca,'XGrid','off','YGrid','off','Fontsize',20);
set(gca,'xtick',[])
set(gca,'xticklabel',[])
set(gca,'ytick',[])
set(gca,'yticklabel','[]')       
